# overlapping gene selection simulation
# sense-antisense overlap  123123
##########                   321321
use strict;   
use warnings; 
use diagnostics;
use autodie;

######----------------input--------------------
my $size_of_set=10;
my $number_of_codons=1000;
my $distance=0.2;
my $dnds_1=2;#shift;
my $dnds_2=0.2;#shift;
my $R=10; ######     This is transition transversion ratio =a/(2b)



###--------------------------main part--------------------------------------
print "distance is $distance, dnds1 is $dnds_1, dnds2 is $dnds_2\n";
my $factor=0.9;  ####This will correct the simulation bias for neutral site
$distance=$distance/$factor;
if ($dnds_1>1) {
	if($dnds_2>1){
		$factor=$factor/$dnds_2/$dnds_1;
	} else{
		$factor=$factor/$dnds_1;
	}
} elsif($dnds_2>1) {
	$factor=$factor/$dnds_2;
}
my @random_DNA=( );
my @mutate_DNA_1;
my @mutate_DNA_2;
my @dnds_2genes_set;
srand (time|$$);  # introduce a random seed each time  

@random_DNA=make_overlap_DNA_set($number_of_codons,$size_of_set);     
@mutate_DNA_1=make_mutation_set($distance,$R,@random_DNA);
@mutate_DNA_2=make_mutation_set($distance,$R,@random_DNA);
@dnds_2genes_set=get_dnds_set($size_of_set,$R,\@mutate_DNA_1,\@mutate_DNA_2);  

print "\ndnds1_by_ns\tdnds2_by_ns\tdnds1_by_ss\tdnds2_by_ss\tvar(dnn)\tvar(dsn)\tvar(dns)\tvar(dss)\tdnn\tdsn\tdns\tdss\tMnn\tMsn\tMns\tMss\tLnn\tLsn\tLns\tLss\tdnds1_mNG\tdnds2_mNG\n";


for (my $i=0; $i<$size_of_set; $i++) {
	for (my $j=0; $j <22;$j++){
		print "$dnds_2genes_set[$j+22*$i]\t";  
	}
	print "\n";
}
exit;

sub get_dnds_set {
	my ($size_of_set,$R,$mutate_DNA_1,$mutate_DNA_2)=@_;
	my @set;
	my @dnds;
	my $dna1;
	my $dna2;
	for (my $i=0; $i < $size_of_set; ++$i) { 
		$dna1=@$mutate_DNA_1[$i];
		$dna2=@$mutate_DNA_2[$i];
		@dnds=calculate_dnds($dna1,$dna2,$R);  
		@set=(@set,@dnds);  
	}
	return @set;	
}


sub calculate_dnds {
	my ($dna1,$dna2,$R)=@_;
	my @N_S=N_S($dna1,$dna2,$R);  
	my @n_s=n_s($dna1,$dna2);  
	my @result;
	my $pn1n2=$n_s[0]/$N_S[0];
	my $ps1n2=$n_s[1]/$N_S[1];
	my $pn1s2=$n_s[2]/$N_S[2];
	my $ps1s2=$n_s[3]/$N_S[3];
	my $dn1n2=-3/4*log(1-4/3*$pn1n2);
	my $ds1n2=-3/4*log(1-4/3*$ps1n2);
	my $dn1s2=-3/4*log(1-4/3*$pn1s2);
	my $ds1s2=-3/4*log(1-4/3*$ps1s2);
	my $pn_1=$n_s[4]/$N_S[4];
	my $ps_1=$n_s[5]/$N_S[5];
	my $pn_2=$n_s[6]/$N_S[6];
	my $ps_2=$n_s[7]/$N_S[7];
	my $dn_1=-3/4*log(1-4/3*$pn_1);
	my $dn_2=-3/4*log(1-4/3*$pn_2);
	my $ds_1=-3/4*log(1-4/3*$ps_1);
	my $ds_2=-3/4*log(1-4/3*$ps_2);
	my $origin_dnds1=$dn_1/$ds_1;
	my $origin_dnds2=$dn_2/$ds_2;
	my $var_pn1n2=$pn1n2*(1-$pn1n2)/$N_S[0];
	my $var_ps1n2=$ps1n2*(1-$ps1n2)/$N_S[1];
	my $var_pn1s2=$pn1s2*(1-$pn1s2)/$N_S[2];
	my $var_ps1s2=$ps1s2*(1-$ps1s2)/$N_S[3];
	my $var_dn1n2=$var_pn1n2/(1-4/3*$pn1n2)**2;
	my $var_ds1n2=$var_ps1n2/(1-4/3*$ps1n2)**2;
	my $var_dn1s2=$var_pn1s2/(1-4/3*$pn1s2)**2;
	my $var_ds1s2=$var_ps1s2/(1-4/3*$ps1s2)**2;
	my $dnds1_1=0;
	my $dnds1_2=0;
	my $dnds2_1=0;
	my $dnds2_2=0;
	my $dnds1;
	my $dnds2;
	if ($ds1n2!=0){
		$dnds1_1=$dn1n2/$ds1n2;
	}
	if ($dn1s2!=0)	{
		$dnds2_1=$dn1n2/$dn1s2;
	}
	if ($ds1s2!=0){
		$dnds1_2=$dn1s2/$ds1s2;
		$dnds2_2=$ds1n2/$ds1s2;
	}
	if (($dnds1_1!=0)and($dnds1_2!=0)){
		$dnds1=($dnds1_1+$dnds1_2)/2;
	} elsif ($dnds1_1!=0){
		$dnds1=$dnds1_1;
	} elsif ($dnds1_2!=0){
		$dnds1=$dnds1_2;
	}
	if (($dnds2_1!=0)and($dnds2_2!=0)){
		$dnds2=($dnds2_1+$dnds2_2)/2;
	} elsif ($dnds2_1!=0){
		$dnds2=$dnds2_1;
	} elsif ($dnds2_2!=0){
		$dnds2=$dnds2_2;
	} 
	@result=($dn1n2/$ds1n2,$dn1n2/$dn1s2,$dn1s2/$ds1s2,$ds1n2/$ds1s2,$var_dn1n2,$var_ds1n2,$var_dn1s2,$var_ds1s2,$dn1n2,$ds1n2,$dn1s2,$ds1s2,$n_s[0],$n_s[1],$n_s[2],$n_s[3],$N_S[0],$N_S[1],$N_S[2],$N_S[3],$origin_dnds1,$origin_dnds2);
	return @result;
}


sub N_S {
	my ($dna1,$dna2,$R)=@_;
	my @result;
	my $N1N2=0;
	my $N1S2=0;
	my $S1N2=0;
	my $S1S2=0;
	my $N1=0;
	my $S1=0;
	my $N2=0;
	my $S2=0;
	my @N_S_1=N_S_dna($dna1,$R); 
	my @N_S_2=N_S_dna($dna2,$R);
	my $length=length($dna1);
	#print "length of dna is $length\n";
	$N1N2=($N_S_1[0]+$N_S_2[0])/2;
	$S1N2=($N_S_1[1]+$N_S_2[1])/2;
	$N1S2=($N_S_1[2]+$N_S_2[2])/2;
	$S1S2=($N_S_1[3]+$N_S_2[3])/2;
	$N1=($N_S_1[4]+$N_S_2[4])/2;
	$S1=($N_S_1[5]+$N_S_2[5])/2;
	$N2=($N_S_1[6]+$N_S_2[6])/2;
	$S2=($N_S_1[7]+$N_S_2[7])/2;
	@result=($N1N2,$S1N2,$N1S2,$S1S2,$N1,$S1,$N2,$S2);
	return @result;
}


sub N_S_dna{  
	my ($dna,$R)=@_;
	my $codon1;
	my $codon2;
	my $position1;
	my $position2;
	my $N1N2=0;
	my $N1S2=0;
	my $S1N2=0;
	my $S1S2=0; 
	my $length=length($dna);
	my @partial_result;
	my @result;
	for ( my $i=2; $i < ($length-2); $i++){ 
		if ($i%3==1){
			$codon1=substr($dna,$i-1,3);
			$codon2=substr($dna,$i-2,3);
			$codon2=reverse($codon2);
			$codon2=~tr/ACGTacgt/TGCAtgca/;
			$position1=1;
			$position2=0;
			@partial_result=possible_change($codon1,$codon2,$position1,$position2,$R);
		} elsif($i%3==2) {
			$codon1=substr($dna,$i-2,3);
			$codon2=substr($dna,$i,3);
			$codon2=reverse($codon2);
			$codon2=~tr/ACGTacgt/TGCAtgca/;
			$position1=2;
			$position2=2;
			@partial_result=possible_change($codon1,$codon2,$position1,$position2,$R);
		} elsif($i%3==0) {
			$codon1=substr($dna,$i,3);
			$codon2=substr($dna,$i-1,3);
			$codon2=reverse($codon2);
			$codon2=~tr/ACGTacgt/TGCAtgca/;
			$position1=0;
			$position2=1;
			@partial_result=possible_change($codon1,$codon2,$position1,$position2,$R);
		}
		$N1N2=$N1N2+$partial_result[0];
		$S1N2=$S1N2+$partial_result[1];
		$N1S2=$N1S2+$partial_result[2];
		$S1S2=$S1S2+$partial_result[3];
	}
	my $N1=$N1N2+$N1S2;
	my $S1=$S1N2+$S1S2;
	my $N2=$N1N2+$S1N2;
	my $S2=$N1S2+$S1S2;
	@result=($N1N2,$S1N2,$N1S2,$S1S2,$N1,$S1,$N2,$S2);
	return @result;
}


sub n_s {   
	my ($dna1,$dna2)=@_;
	my $n1n2=0;
	my $s1n2=0;
	my $n1s2=0;
	my $s1s2=0;
	my $n1n2_pathway=0;
	my $s1n2_pathway=0;
	my $n1s2_pathway=0;
	my $s1s2_pathway=0;
	my $region1=substr($dna1,0,3);
	my $region2=substr($dna2,0,3);
	my @place_change;
	my @region_change;
	my @partial_result=(0,0,0,0);
	my @result;
	my $pathway_number=0;
	my $length=length($dna1);
	my $i=0;
	while ($i < ($length-2)) {  
		@region_change=switch_region($region1,$region2,$i,$dna1,$dna2);
		$region1=$region_change[0];
		$region2=$region_change[1];
		$i=$region_change[2]; ##
		@partial_result=pathway($region1,$region2,$i,$pathway_number,$n1n2_pathway,$s1n2_pathway,$n1s2_pathway,$s1s2_pathway,$n1n2_pathway,$s1n2_pathway,$n1s2_pathway,$s1s2_pathway);
		if ($i%3==0) {
			if ((length($region1)%3)==0) {
				$i=$i+length($region1)-1; 
			} elsif((length($region1)%3)==2) {
				$i=$i+length($region1)-2;
			}
		} elsif($i%3==2) {
			if ( (length($region1)%3)==0){
				$i=$i+length($region1)-2;
			} elsif((length($region1)%3)==1) {
				$i=$i+length($region1)-1;
			}
		}
		if ($i<($length-2)){
			$region1=substr($dna1,$i,3);
			$region2=substr($dna2,$i,3);
		} 
		if ($partial_result[8]>0){
		$n1n2=$n1n2+$partial_result[0]/$partial_result[8];
		$s1n2=$s1n2+$partial_result[1]/$partial_result[8];
		$n1s2=$n1s2+$partial_result[2]/$partial_result[8];
		$s1s2=$s1s2+$partial_result[3]/$partial_result[8];
		}
	}
	my $n1=$n1n2+$n1s2;
	my $s1=$s1n2+$s1s2;
	my $n2=$n1n2+$s1n2;
	my $s2=$n1s2+$s1s2;
	@result=($n1n2,$s1n2,$n1s2,$s1s2,$n1,$s1,$n2,$s2);
	return @result;
}

sub pathway{  
	my ($region1,$region2,$i,$pathway_number,$n1n2,$s1n2,$n1s2,$s1s2,$n1n2_pathway,$s1n2_pathway,$n1s2_pathway,$s1s2_pathway)=@_;
	my $change_number=change_number($region1,$region2);
	my @change_place=change_place($region1,$region2);
	my $place_region;
	my $base;
	my $new_region1;
	my $codon1;
	my $new_codon1;
	my @partial_result;
	my @result;
	my @set;
	my $codon1_region1;
	my $codon1_region2;
	my $codon2_region1;
	my $codon2_region2;
	my $aa1_region1;
	my $aa2_region1;
	my $aa1_region2;
	my $aa2_region2;
	my $length=length($region1);
	for (my $j=0;$j<$change_number;$j++){
		$place_region=$change_place[$j];
		$new_region1=$region1;
		$base=substr($region2,$place_region,1);
		substr($new_region1,$place_region,1,$base);
		if ($i%3==0){ 
			if ($place_region%3==1){
				$codon1_region1=substr($region1,$place_region-1,3); 
				$codon2_region1=substr($region1,$place_region-2,3);
				$codon2_region1=reverse($codon2_region1); 
				$codon2_region1=~tr/ACGTacgt/TGCAtgca/;
				$codon1_region2=substr($new_region1,$place_region-1,3);
				$codon2_region2=substr($new_region1,$place_region-2,3);
				$codon2_region2=reverse($codon2_region2);
				$codon2_region2=~tr/ACGTacgt/TGCAtgca/;
				$aa1_region1=codon2aa($codon1_region1);
				$aa2_region1=codon2aa($codon2_region1);
				$aa1_region2=codon2aa($codon1_region2);
				$aa2_region2=codon2aa($codon2_region2);
				@partial_result=compare_dna1_dna2($aa1_region1,$aa2_region1,$aa1_region2,$aa2_region2); 
			} elsif($place_region%3==2) {
				$codon1_region1=substr($region1,$place_region-2,3); 
				$codon2_region1=substr($region1,$place_region,3); 
				$codon2_region1=reverse($codon2_region1);
				$codon2_region1=~tr/ACGTacgt/TGCAtgca/;
				$codon1_region2=substr($new_region1,$place_region-2,3);  
				$codon2_region2=substr($new_region1,$place_region,3);    
				$codon2_region2=reverse($codon2_region2); 
				$codon2_region2=~tr/ACGTacgt/TGCAtgca/;
				$aa1_region1=codon2aa($codon1_region1);
				$aa2_region1=codon2aa($codon2_region1);
				$aa1_region2=codon2aa($codon1_region2);
				$aa2_region2=codon2aa($codon2_region2);
				@partial_result=compare_dna1_dna2($aa1_region1,$aa2_region1,$aa1_region2,$aa2_region2);
			} elsif($place_region%3==0) {
				$codon1_region1=substr($region1,$place_region,3);
				$codon2_region1=substr($region1,$place_region-1,3); 
				$codon2_region1=reverse($codon2_region1);
				$codon2_region1=~tr/ACGTacgt/TGCAtgca/;
				$codon1_region2=substr($new_region1,$place_region,3);
				$codon2_region2=substr($new_region1,$place_region-1,3);
				$codon2_region2=reverse($codon2_region2);
				$codon2_region2=~tr/ACGTacgt/TGCAtgca/;
				$aa1_region1=codon2aa($codon1_region1);
				$aa2_region1=codon2aa($codon2_region1);
				$aa1_region2=codon2aa($codon1_region2);
				$aa2_region2=codon2aa($codon2_region2);
				@partial_result=compare_dna1_dna2($aa1_region1,$aa2_region1,$aa1_region2,$aa2_region2);
			}
		}
		
		if ($i%3==2){
			if ($place_region%3==1){
				$codon1_region1=substr($region1,$place_region,3);  
				$codon2_region1=substr($region1,$place_region-1,3);
				$codon2_region1=reverse($codon2_region1);
				$codon2_region1=~tr/ACGTacgt/TGCAtgca/;
				$codon1_region2=substr($new_region1,$place_region,3);
				$codon2_region2=substr($new_region1,$place_region-1,3);
				$codon2_region2=reverse($codon2_region2);
				$codon2_region2=~tr/ACGTacgt/TGCAtgca/;
				$aa1_region1=codon2aa($codon1_region1);
				$aa2_region1=codon2aa($codon2_region1);
				$aa1_region2=codon2aa($codon1_region2);
				$aa2_region2=codon2aa($codon2_region2);
				@partial_result=compare_dna1_dna2($aa1_region1,$aa2_region1,$aa1_region2,$aa2_region2);
			} elsif($place_region%3==2) {
				$codon1_region1=substr($region1,$place_region-1,3);
				$codon2_region1=substr($region1,$place_region-2,3);
				$codon2_region1=reverse($codon2_region1);
				$codon2_region1=~tr/ACGTacgt/TGCAtgca/;
				$codon1_region2=substr($new_region1,$place_region-1,3);
				$codon2_region2=substr($new_region1,$place_region-2,3);
				$codon2_region2=reverse($codon2_region2);
				$codon2_region2=~tr/ACGTacgt/TGCAtgca/;
				$aa1_region1=codon2aa($codon1_region1);
				$aa2_region1=codon2aa($codon2_region1);
				$aa1_region2=codon2aa($codon1_region2);
				$aa2_region2=codon2aa($codon2_region2);
				@partial_result=compare_dna1_dna2($aa1_region1,$aa2_region1,$aa1_region2,$aa2_region2);
			} elsif($place_region%3==0) {
				$codon1_region1=substr($region1,$place_region-2,3);
				$codon2_region1=substr($region1,$place_region,3);
				$codon2_region1=reverse($codon2_region1);
				$codon2_region1=~tr/ACGTacgt/TGCAtgca/;
				$codon1_region2=substr($new_region1,$place_region-2,3);
				$codon2_region2=substr($new_region1,$place_region,3);
				$codon2_region2=reverse($codon2_region2);
				$codon2_region2=~tr/ACGTacgt/TGCAtgca/;
				$aa1_region1=codon2aa($codon1_region1);
				$aa2_region1=codon2aa($codon2_region1);
				$aa1_region2=codon2aa($codon1_region2);
				$aa2_region2=codon2aa($codon2_region2);
				@partial_result=compare_dna1_dna2($aa1_region1,$aa2_region1,$aa1_region2,$aa2_region2);
			}
		}
		
		if (($aa1_region2 ne '_') & ($aa2_region2 ne '_')){
			$n1n2_pathway=$n1n2_pathway+$partial_result[0];   
			$s1n2_pathway=$s1n2_pathway+$partial_result[1];
			$n1s2_pathway=$n1s2_pathway+$partial_result[2];
			$s1s2_pathway=$s1s2_pathway+$partial_result[3];
			if ($change_number >1) {
				$region1=$new_region1;
				@result=pathway($region1,$region2,$i,$pathway_number,$n1n2,$s1n2,$n1s2,$s1s2,$n1n2_pathway,$s1n2_pathway,$n1s2_pathway,$s1s2_pathway);
				$n1n2=$result[0];
				$s1n2=$result[1];
				$n1s2=$result[2];
				$s1s2=$result[3];
				$pathway_number=$result[8];
			} else {
				$pathway_number++;
				$n1n2=$n1n2+$n1n2_pathway;
				$s1n2=$s1n2+$s1n2_pathway;
				$n1s2=$n1s2+$n1s2_pathway;
				$s1s2=$s1s2+$s1s2_pathway;
				$n1n2_pathway=0;
				$s1n2_pathway=0;
				$n1s2_pathway=0;
				$s1s2_pathway=0;
			} 
		} else{
			$n1n2_pathway=0;
			$s1n2_pathway=0;
			$n1s2_pathway=0;
			$s1s2_pathway=0;
		}
	}
	@set=($n1n2,$s1n2,$n1s2, $s1s2,$n1n2_pathway,$s1n2_pathway,$n1s2_pathway,$s1s2_pathway,$pathway_number);
	return @set;
}

sub switch_region {  ###
	my ($region1,$region2,$i,$dna1,$dna2)=@_; #$i is where region with mutation starts in $dna1
	my @result;
	my $change_number;
	my $length=length($dna1);  
	my $new_region1;
	my $new_region2;
	my $j;
	my $new_change_number;
	my @compare_result;
	$change_number=change_number($region1,$region2);
	while (($change_number==0) & ($i<($length-3))) {#####
		if (($i%3)==0) {
			$i=$i+2;
			if ($i<($length-2)){
				$region1=substr($dna1,$i,3);
				$region2=substr($dna2,$i,3);
			}
		} elsif (($i%3)==2) {
			$i=$i+1;
			if ($i<($length-2)){
				$region1=substr($dna1,$i,3);
				$region2=substr($dna2,$i,3);
			}
		}
		$change_number=change_number($region1,$region2);
	}
	if ($change_number>0){
		$j=$i+3;
		if ($i%3==0){ 
			if ($j<($length-1)){ ##
				$new_region1=substr($dna1,$j,2);
				$new_region2=substr($dna2,$j,2);
				$new_change_number=change_number($new_region1,$new_region2);
				@compare_result=compare_new_region($j,$new_region1,$new_region2,$dna1,$dna2);
			}
		}
		if ($i%3==2) { ##
			if ($j<$length){###
				$new_region1=substr($dna1,$j,1);
				$new_region2=substr($dna2,$j,1);
				$new_change_number=change_number($new_region1,$new_region2);
				@compare_result=compare_new_region($j,$new_region1,$new_region2,$dna1,$dna2);
			}
		}	
	}
	if (@compare_result ne 0) {
		$region1=$region1.$compare_result[0];
		$region2=$region2.$compare_result[1];
	}
	@result=($region1,$region2,$i); ##
}

sub compare_new_region {  ##
	my ($j,$add_region1,$add_region2,$dna1,$dna2)=@_;
	my $change; 
	my $try_region1;
	my $try_region2;
	my $length=length($dna1);
	my @result;
	$change=change_number($add_region1,$add_region2);  
	while (($change>0) & ($j<$length-1)) {#########
		if (($j%3)==0) {  
			$try_region1=substr($dna1,$j+2,1);   
			$try_region2=substr($dna2,$j+2,1);
			$change=change_number($try_region1,$try_region2);
			if ($change> 0) {
				$j=$j+2;
			}
			$add_region1=$add_region1.$try_region1;
			$add_region2=$add_region2.$try_region2;
		}
		if (($j%3)==2) {   
			$try_region1=substr($dna1,$j+1,2);
			$try_region2=substr($dna2,$j+1,2);
			$change=change_number($try_region1,$try_region2);
			if ($change> 0) {
				$j=$j+1;
			}
			$add_region1=$add_region1.$try_region1;
			$add_region2=$add_region2.$try_region2;
		}
	}
	@result=($add_region1,$add_region2);
	return @result;
}

sub change_number {  ##
	my ($region1,$region2)=@_;
	my $number=0;
	my $base1;
	my $base2;
	for (my $i=0; $i<length($region1);$i++){
		$base1=substr($region1,$i,1);
		$base2=substr($region2,$i,1);
		if ($base1 ne $base2) {
			$number++;
		}
	}
	return $number;
}

sub change_place {  ##
	my ($region1,$region2)=@_;
	my @result;
	my $base1;
	my $base2;
	for (my $i=0; $i<length($region1);$i++){
		$base1=substr($region1,$i,1);
		$base2=substr($region2,$i,1);
		if ($base1 ne $base2){  
			push(@result,$i);
		}
	}
	return @result;  ###
}

sub compare_dna1_dna2 {
	my ($aa1_dna1,$aa2_dna1,$aa1_dna2,$aa2_dna2)=@_;
	my @result=(0,0,0,0);
	if ($aa1_dna1 eq $aa1_dna2) {
		if ($aa2_dna1 eq $aa2_dna2) {
			$result[3]=1;  
		} else {
			$result[1]=1;
		}
	} elsif ($aa2_dna1 eq $aa2_dna2){
		$result[2]=1;
	} else{
		$result[0]=1;
	}
	return @result;
}

sub possible_change {
	my ($codon1,$codon2,$position1,$position2,$R)=@_;
	my $original_base=substr($codon1,$position1,1);
	my $new_codon1=$codon1;
	my $new_codon2=$codon2;
	my $aa1=codon2aa($codon1);
	my $aa2=codon2aa($codon2);
	my $new_aa1;
	my $new_aa2;
	my $N1N2=0;
	my $N1S2=0;
	my $S1N2=0;
	my $S1S2=0;
	my $sum=0;
	if ($original_base ne 'A') {
		substr($new_codon1,$position1,1,'A');
		substr($new_codon2,$position2,1,'T');
		$new_aa1=codon2aa($new_codon1);
		$new_aa2=codon2aa($new_codon2);
		if ($aa1 eq $new_aa1) {
			if ($aa2 eq $new_aa2) {
				if ($original_base eq 'T'){
					$S1S2=$S1S2+$R/(1+$R);
				} else{
					$S1S2=$S1S2+1/(2+2*$R);
				}
			} elsif($new_aa1 ne '_') {
				if ($original_base eq 'T'){
					$S1N2=$S1N2+$R/(1+$R);
				} else{
					$S1N2=$S1N2+1/(2+2*$R);
				}
			}
		} elsif($new_aa1 ne '_') {
			if ($aa2 eq $new_aa2) {
				if ($original_base eq 'T'){
					$N1S2=$N1S2+$R/(1+$R);
				} else{
					$N1S2=$N1S2+1/(2+2*$R);
				}
			} elsif($new_aa2 ne '_') {
				if ($original_base eq 'T'){
					$N1N2=$N1N2+$R/(1+$R);
				} else{
					$N1N2=$N1N2+1/(2+2*$R);
				}
			}
		}
	}
	if ($original_base ne 'T') {
		substr($new_codon1,$position1,1,'T');
		substr($new_codon2,$position2,1,'A');
		$new_aa1=codon2aa($new_codon1);
		$new_aa2=codon2aa($new_codon2);
		if ($aa1 eq $new_aa1) {
			if ($aa2 eq $new_aa2) {
				if ($original_base eq 'A'){
					$S1S2=$S1S2+$R/(1+$R);
				} else{
					$S1S2=$S1S2+1/(2+2*$R);
				}
			} elsif($new_aa1 ne '_') {
				if ($original_base eq 'A'){
					$S1N2=$S1N2+$R/(1+$R);
				} else{
					$S1N2=$S1N2+1/(2+2*$R);
				}
			}
		} elsif($new_aa1 ne '_') {
			if ($aa2 eq $new_aa2) {
				if ($original_base eq 'A'){
					$N1S2=$N1S2+$R/(1+$R);
				} else{
					$N1S2=$N1S2+1/(2+2*$R);
				}
			} elsif($new_aa2 ne '_') {
				if ($original_base eq 'A'){
					$N1N2=$N1N2+$R/(1+$R);
				} else{
					$N1N2=$N1N2+1/(2+2*$R);
				}
			}
		}
	}
	if ($original_base ne 'G') {
		substr($new_codon1,$position1,1,'G');
		substr($new_codon2,$position2,1,'C');
		$new_aa1=codon2aa($new_codon1);
		$new_aa2=codon2aa($new_codon2);
		if ($aa1 eq $new_aa1) {
			if ($aa2 eq $new_aa2) {
				if ($original_base eq 'C'){
					$S1S2=$S1S2+$R/(1+$R);
				} else{
					$S1S2=$S1S2+1/(2+2*$R);
				}
			} elsif($new_aa1 ne '_') {
				if ($original_base eq 'C'){
					$S1N2=$S1N2+$R/(1+$R);
				} else{
					$S1N2=$S1N2+1/(2+2*$R);
				}
			}
		} elsif($new_aa1 ne '_') {
			if ($aa2 eq $new_aa2) {
				if ($original_base eq 'C'){
					$N1S2=$N1S2+$R/(1+$R);
				} else{
					$N1S2=$N1S2+1/(2+2*$R);
				}
			} elsif($new_aa2 ne '_') {
				if ($original_base eq 'C'){
					$N1N2=$N1N2+$R/(1+$R);
				} else{
					$N1N2=$N1N2+1/(2+2*$R);
				}
			}
		}
	}
	if ($original_base ne 'C') {
		substr($new_codon1,$position1,1,'C');
		substr($new_codon2,$position2,1,'G');
		$new_aa1=codon2aa($new_codon1);
		$new_aa2=codon2aa($new_codon2);
		if ($aa1 eq $new_aa1) {
			if ($aa2 eq $new_aa2) {
				if ($original_base eq 'G'){
					$S1S2=$S1S2+$R/(1+$R);
				} else{
					$S1S2=$S1S2+1/(2+2*$R);
				}
			} elsif($new_aa1 ne '_') {
				if ($original_base eq 'G'){
					$S1N2=$S1N2+$R/(1+$R);
				} else{
					$S1N2=$S1N2+1/(2+2*$R);
				}
			}
		} elsif($new_aa1 ne '_') {
			if ($aa2 eq $new_aa2) {
				if ($original_base eq 'G'){
					$N1S2=$N1S2+$R/(1+$R);
				} else{
					$N1S2=$N1S2+1/(2+2*$R);
				}
			} elsif($new_aa2 ne '_') {
				if ($original_base eq 'G'){
					$N1N2=$N1N2+$R/(1+$R);
				} else{
					$N1N2=$N1N2+1/(2+2*$R);
				}
			}
		}
	}
	$sum=$N1N2+$N1S2+$S1N2+$S1S2;
	if ($sum ne 0) {
		$N1N2=$N1N2/$sum;
		$N1S2=$N1S2/$sum;
		$S1N2=$S1N2/$sum;
		$S1S2=$S1S2/$sum;
	}
	my @result=($N1N2,$S1N2,$N1S2,$S1S2);
	return @result;
}


sub make_overlap_DNA_set {
	my($number_of_codons,$size_of_set) = @_;     
	my $dna;
	my @set;
	for (my $i = 0; $i < $size_of_set ; ++$i) {
		$dna = make_overlap_DNA ( $number_of_codons ); 
		push(@set, $dna );
	}
	return @set;
}


sub make_mutation_set{
	my ($distance,$R,@random_DNA)=@_;
	my @set;
	foreach my $dna (@random_DNA){  
		$dna=mutation_and_selection($dna,$distance,$R);  
		push(@set,$dna);
	}
	return @set;
}


sub mutation_and_selection{
	my ($dna,$distance,$R)=@_;
	my $newbase;
	my $select_base;
	my $oldbase;
	my $random_number;
	my $position;  
	my $mutate_time=int(length($dna)*$distance/2);
	for (my $i = 0; $i < $mutate_time ; ++$i) {
		$position=randomposition($dna);
		$oldbase=substr($dna,$position,1);
		$random_number=rand(1);
		if ($oldbase eq 'A') {   #####
			if ($random_number < $R/(1+$R)) {
				$newbase='T';
			} elsif ($random_number < (2*$R+1)/(2+2*$R)){
				$newbase='C';
			} else{
				$newbase='G';
			}
		} elsif ($oldbase eq 'T') {
			if ($random_number < $R/(1+$R)) {
				$newbase='A';
			} elsif ($random_number < (2*$R+1)/(2+2*$R)){
				$newbase='C';
			} else{
				$newbase='G';
			}
		} elsif ($oldbase eq 'C') {
			if ($random_number < $R/(1+$R)) {
				$newbase='G';
			} elsif ($random_number < (2*$R+1)/(2+2*$R)){
				$newbase='A';
			} else{
				$newbase='T';
			}
		} elsif ($oldbase eq 'G') {
			if ($random_number < $R/(1+$R)) {
				$newbase='C';
			} elsif ($random_number < (2*$R+1)/(2+2*$R)){
				$newbase='A';
			} else{
				$newbase='T';
			}
		}
		$select_base=selection($dna,$newbase,$position);    
		substr($dna,$position,1,$select_base); 
	}
	return $dna;
}


sub selection{
	my ($dna,$newbase,$position)=@_;
	my $codon_original_1;
	my $codon_mutation_1;
	my $codon_original_2;
	my $codon_mutation_2;
	my $stop_codon='_';
	my $original_aa1;
	my $original_aa2;
	my $mutate_aa1;
	my $mutate_aa2;
	my $select_base=substr($dna,$position,1);
	if ($position%3==2){
		$codon_original_1=substr($dna,$position-2,3);
		$codon_original_2=substr($dna,$position,3);
		$codon_mutation_1=$codon_original_1;  
		$codon_mutation_2=$codon_original_2;
		$codon_original_2=reverse($codon_original_2);
		$codon_original_2=~tr/ACGTacgt/TGCAtgca/;
		substr($codon_mutation_1,2,1,$newbase); 
		substr($codon_mutation_2,0,1,$newbase); 
		$codon_mutation_2=reverse($codon_mutation_2);
		$codon_mutation_2=~tr/ACGTacgt/TGCAtgca/;
	} elsif($position%3==0){
		$codon_original_1=substr($dna,$position,3);
		$codon_original_2=substr($dna,$position-1,3);
		$codon_mutation_1=$codon_original_1;  
		$codon_mutation_2=$codon_original_2;
		$codon_original_2=reverse($codon_original_2);
		$codon_original_2=~tr/ACGTacgt/TGCAtgca/;
		substr($codon_mutation_1,0,1,$newbase); 
		substr($codon_mutation_2,1,1,$newbase); 
		$codon_mutation_2=reverse($codon_mutation_2);
		$codon_mutation_2=~tr/ACGTacgt/TGCAtgca/;
	} elsif($position%3==1){
		$codon_original_1=substr($dna,$position-1,3);
		$codon_original_2=substr($dna,$position-2,3);
        $codon_mutation_1=$codon_original_1;  
		$codon_mutation_2=$codon_original_2;
		$codon_original_2=reverse($codon_original_2);
		$codon_original_2=~tr/ACGTacgt/TGCAtgca/;
		substr($codon_mutation_1,1,1,$newbase); 
		substr($codon_mutation_2,2,1,$newbase); 
		$codon_mutation_2=reverse($codon_mutation_2);
		$codon_mutation_2=~tr/ACGTacgt/TGCAtgca/;
	}
	$original_aa1=codon2aa($codon_original_1);
	$original_aa2=codon2aa($codon_original_2);
	$mutate_aa1=codon2aa($codon_mutation_1);
	$mutate_aa2=codon2aa($codon_mutation_2);	
	if (($original_aa1 eq $mutate_aa1)){
		if(($original_aa2 eq $mutate_aa2)){
			if (rand(1)<=$factor){
				$select_base=$newbase;
			}
		} elsif(( rand(1) <= $factor*$dnds_2) & ($mutate_aa2 ne $stop_codon)){
			$select_base=$newbase;
		} 
	} elsif(($mutate_aa1 ne $stop_codon)) { 
		if (($original_aa2 eq $mutate_aa2)){
			if( rand(1) <= $factor*$dnds_1){
				$select_base=$newbase;
			}
		} elsif((rand(1) <= $factor*$dnds_2*$dnds_1) & ($mutate_aa2 ne $stop_codon)){
			$select_base=$newbase;
		}
	}
	return $select_base;
}


sub randomposition{
	my($string)= @_;
	return (int(rand(length($string)-4))+2);  
}


sub make_overlap_DNA{ #
	my ($number_of_codons)= @_;
	my $dna;
	my $codon;
	my $stop_codon='_';
	for(my $i=0;$i<$number_of_codons;++$i){  
		$dna.=random_codon();
	}
	for(my $i=2;$i < (length($dna)-2);){
		$codon=substr($dna,$i,3);
		$codon=reverse($codon);
		$codon=~tr/ACGTacgt/TGCAtgca/;
		if (codon2aa($codon) eq $stop_codon) {  
			substr($dna,$i,3)="";
			$codon=substr($dna,$i-2,3);
			if (codon2aa($codon) eq $stop_codon) {
				substr($dna,$i-2,3)="";
			}
		} else {
			$i=$i+3;
		}
	}
	return $dna;
}


sub randomnucleotide{    
	my(@nucleotides)=('A','C','G','T');
	return randomelement(@nucleotides);
}


sub random_codon{
	my(@codons)=('TCA','TCC','TCG','TCT','TTC','TTT','TTA','TTG','TAC','TAT','TGC','TGT','TGG','CTA','CTC','CTG','CTT','CCA','CCC','CCG','CCT','CAC','CAT','CAA','CAG','CGA','CGC','CGG','CGT','ATA','ATC','ATT','ATG','ACA','ACC','ACG','ACT','AAC','AAT','AAA','AAG','AGC','AGT','AGA','AGG','GTA','GTC','GTG','GTT','GCA','GCC','GCG','GCT','GAC','GAT','GAA','GAG','GGA','GGC','GGG','GGT');
	return randomelement(@codons);
}


sub randomelement{
	my(@array)= @_;
	return $array[rand @array];
}


sub codon2aa {
my($codon) = @_;
$codon = uc $codon;    # uc makes the entire string uppercase
my(%genetic_code) = (
'TCA' => 'S', # Serine
'TCC' => 'S', # Serine
'TCG' => 'S', # Serine
'TCT' => 'S', # Serine
'TTC' => 'F', # Phenylalanine
'TTT' => 'F', # Phenylalanine
'TTA' => 'L', # Leucine
'TTG' => 'L', # Leucine
'TAC' => 'Y', # Tyrosine
'TAT' => 'Y', # Tyrosine
'TAA' => '_', # Stop
'TAG' => '_', # Stop
'TGC' => 'C', # Cysteine
'TGT' => 'C', # Cysteine
'TGA' => '_', # Stop
'TGG' => 'W', # Tryptophan
'CTA' => 'L', # Leucine
'CTC' => 'L', # Leucine
'CTG' => 'L', # Leucine
'CTT' => 'L', # Leucine
'CCA' => 'P', # Proline
'CCC' => 'P', # Proline
'CCG' => 'P', # Proline
'CCT' => 'P', # Proline
'CAC' => 'H', # Histidine
'CAT' => 'H', # Histidine
'CAA' => 'Q', # Glutamine
'CAG' => 'Q', # Glutamine
'CGA' => 'R', # Arginine
'CGC' => 'R', # Arginine
'CGG' => 'R', # Arginine
'CGT' => 'R', # Arginine
'ATA' => 'I', # Isoleucine
'ATC' => 'I', # Isoleucine
'ATT' => 'I', # Isoleucine
'ATG' => 'M', # Methionine
'ACA' => 'T', # Threonine
'ACC' => 'T', # Threonine
'ACG' => 'T', # Threonine
'ACT' => 'T', # Threonine
'AAC' => 'N', # Asparagine
'AAT' => 'N', # Asparagine
'AAA' => 'K', # Lysine
'AAG' => 'K', # Lysine
'AGC' => 'S', # Serine
'AGT' => 'S', # Serine
'AGA' => 'R', # Arginine
'AGG' => 'R', # Arginine
'GTA' => 'V', # Valine
'GTC' => 'V', # Valine
'GTG' => 'V', # Valine
'GTT' => 'V', # Valine
'GCA' => 'A', # Alanine
'GCC' => 'A', # Alanine
'GCG' => 'A', # Alanine
'GCT' => 'A', # Alanine
'GAC' => 'D', # Aspartic Acid
'GAT' => 'D', # Aspartic Acid
'GAA' => 'E', # Glutamic Acid
'GAG' => 'E', # Glutamic Acid
'GGA' => 'G', # Glycine
'GGC' => 'G', # Glycine
'GGG' => 'G', # Glycine
'GGT' => 'G', # Glycine
);
if(exists $genetic_code{$codon}) {
	return $genetic_code{$codon};
	}else{
	print STDERR "Bad codon \"$codon\"!!\n";
	exit;
}
}



